@@ -19,6 +19,8 @@ from .modelhandler import ModelHandler
 from .mlhandler import MLHandler
 from .filterhandler import FilterHandler
 from .drivehandler import DriveHandler
+from .comichandler import ComicHandler
+from .openapihandler import OpenAPIHandler
 
 # Aliases
 Command = ProcessHandler
@@ -34,6 +36,8 @@ Slide = PPTXHandler
 Storage = DriveHandler
 Twitter = TwitterRESTHandler
 Websocket = WebSocketHandler
+Comic = ComicHandler
+OpenAPI = OpenAPIHandler
 
 
 __all__ = [
@@ -42,6 +46,8 @@ __all__ = [
     'Capture',
     'CaptureHandler',
     'Command',
+    'Comic',
+    'ComicHandler',
     'Data',
     'DirectoryHandler',
     'DriveHandler',
@@ -59,6 +65,8 @@ __all__ = [
     'ModelHandler',
     'ML',
     'MLHandler',
+    'OpenAPI',
+    'OpenAPIHandler',
     'PPTXHandler',
     'ProcessHandler',
     'Proxy',
