@@ -124,7 +124,8 @@ class ProcessHandler(BaseHandler):
 
         proc = Subprocess(
             self.cmdargs,
-            shell=self.shell,
+            # NOTE: developer should sanitize args if shell=True
+            shell=self.shell,  # nosec: developer-initiated
             cwd=self.cwd,
             stream_stdout=self.stream_stdout,
             stream_stderr=self.stream_stderr,
