@@ -107,7 +107,7 @@ class SocialHandler(BaseHandler):
         info = self.session.get(self.user_info, {})
         token = self.kwargs.get(key, None)         # Get from config
         session_token = fetch(info, key, None)
-        if token == 'persist':                          # nosec
+        if token == 'persist':                          # nosec: false positive
             token = self.read_store().get(key, None)    # If persist, use store
             if token is None and session_token:         # Or persist from session
                 self.write_store(info)
