@@ -20,7 +20,8 @@ from PIL import Image
 from functools import partial
 from gramex.config import app_log, objectpath
 from gramex.transforms import build_transform
-from lxml.html import fragments_fromstring, builder, HtmlElement
+# lxml.etree is safe on https://github.com/tiran/defusedxml/tree/main/xmltestdata
+from lxml.html import fragments_fromstring, builder, HtmlElement    # nosec: lxml is safe
 from orderedattrdict import AttrDict
 from pptx.chart import data as pptxchartdata
 from pptx.dml.color import RGBColor
