@@ -3,7 +3,6 @@ import datetime
 import importlib
 import json
 import os
-import six
 import time
 import tornado.gen
 import yaml
@@ -23,7 +22,7 @@ def _arg_repr(arg):
     values are treated as strings. For example, ``=x`` is the variable ``x`` but
     ``x`` is the string ``"x"``. ``==x`` is the string ``"=x"``.
     '''
-    if isinstance(arg, six.string_types):
+    if isinstance(arg, str):
         if arg.startswith('=='):
             return repr(arg[1:])        # "==x" becomes '"=x"'
         elif arg.startswith('='):
@@ -178,7 +177,7 @@ def build_transform(conf, vars=None, filename='transform', cache=False, iter=Tru
         raise KeyError('%s: No function in conf %s' % (filename, conf))
 
     # Get the name of the function in case it's specified as a function call
-    # expr is the full function / expression, e.g. six.text_type("abc")
+    # expr is the full function / expression, e.g. str("abc")
     # tree is the ast result
     expr = conf['function']
     tree = ast.parse(expr)
@@ -191,12 +190,14 @@ def build_transform(conf, vars=None, filename='transform', cache=False, iter=Tru
     # Else, use the expression as-is
     function_name = _full_name(tree.body[0].value)
     module_name = function_name.split('.')[0] if isinstance(function_name, str) else None
+    function, doc = None, expr
     # If the module or function is one of the vars themselves, return it as-is
     # _val.type will be used as-is, then, rather than looking for an "_val" module
     if module_name in vars:
         expr = function_name
     elif function_name is not None:
         function = locate(function_name, modules=['gramex.transforms'])
+        doc = function.__doc__
         if function is None:
             app_log.error('%s: Cannot load function %s' % (filename, function_name))
         # This section converts the function into an expression.
@@ -217,6 +218,10 @@ def build_transform(conf, vars=None, filename='transform', cache=False, iter=Tru
             for key, val in conf.get('kwargs', {}).items():
                 expr += '%s=%s, ' % (key, _arg_repr(val))
             expr += ')'
+    # If expr starts with a function call (e.g. module.func(...)), use it's docs
+    elif isinstance(tree.body[0].value, ast.Call):
+        from astor import to_source
+        doc = locate(to_source(tree.body[0].value.func).strip()).__doc__
 
     # Create the code
     modules = module_names(tree, vars)
@@ -243,14 +248,16 @@ def build_transform(conf, vars=None, filename='transform', cache=False, iter=Tru
         **{key: getattr(gramex.transforms, key) for key in gramex.transforms.__all__}
     )
     code = compile(''.join(body), filename=filename, mode='exec')
-    exec(code, context)         # nosec - OK to run arbitrary Python code in YAML
+    # exec() is safe here since the code is written by app developer
+    exec(code, context)     # nosec: developer-initiated
 
     # Return the transformed function
-    function = context['transform']
-    function.__name__ = str(function_name or filename)
-    function.__doc__ = str(function.__doc__)
+    result = context['transform']
+    result.__name__ = str(function_name or filename)
+    result.__doc__ = str(doc)
+    result.__func__ = function
 
-    return function
+    return result
 
 
 def condition(*args):
@@ -281,8 +288,9 @@ def condition(*args):
     else:
         pairs = zip(args[0::2], args[1::2])
     for cond, val in pairs:
-        if isinstance(cond, six.string_types):
-            if eval(Template(cond).substitute(var_defaults)):    # nosec - any Python expr is OK
+        if isinstance(cond, str):
+            # eval() is safe here since `cond` is written by app developer
+            if eval(Template(cond).substitute(var_defaults)):    # nosec: developer-initiated
                 return val
         elif bool(cond):
             return val
@@ -331,10 +339,10 @@ def flattener(fields, default=None, filename='flatten'):
             body.append('\tr[%s] = %s\n' % (field, target))
 
     for field, source in fields.items():
-        if not isinstance(field, six.string_types):
+        if not isinstance(field, str):
             app_log.error('flattener:%s: key %s is not a str', filename, field)
             continue
-        if isinstance(source, six.string_types):
+        if isinstance(source, str):
             target = 'obj'
             if source:
                 for item in source.split('.'):
@@ -350,7 +358,8 @@ def flattener(fields, default=None, filename='flatten'):
     body.append('\treturn r')
     code = compile(''.join(body), filename='flattener:%s' % filename, mode='exec')
     context = {'AttrDict': AttrDict, 'default': default}
-    eval(code, context)     # nosec - code constructed entirely in this function
+    # eval() is safe here since the code is constructed entirely in this function
+    eval(code, context)     # nosec: developer-initiated
     return context[filename]
 
 
@@ -391,18 +400,30 @@ _convert_map = {
 }
 
 
-def convert(hint, *args):
+def typelist(hint):
+    typ, is_list = hint, False
+    # If typ is an Annotation, use the native type. Else use the type.
+    while hasattr(typ, '__args__'):
+        is_list = getattr(typ, '_name', None) in ('List', 'Tuple')
+        typ = typ.__args__[0]
+    return typ, is_list
+
+
+def convert(hint, param, args):
     from pandas.core.common import flatten
     args = list(flatten(args))
-    # If hint is List[int], Tuple[int], etc. then return a list or a tuple after type conversion
-    #   hint.__args__ = (int, )
-    #   hint.__origin__ = list or tuple
-    if hasattr(hint, '__args__'):
-        method = _convert_map.get(hint.__args__[0], hint.__args__[0])
-        return hint.__origin__(map(method, args))
-    # Otherwise, just pick the LAST value and return it
-    method = _convert_map.get(hint, hint)
-    return method(args[-1])
+    typ, is_list = typelist(hint)
+    # Convert args to the native type
+    method = _convert_map.get(typ, typ)
+    # If default is list-like or hint is List-like, return list. Else return last value
+    if is_list or isinstance(param.default, (list, tuple)):
+        return [method(arg) for arg in args]
+    else:
+        return method(args[-1] if len(args) else param.default)
+
+
+class Header(object):
+    pass
 
 
 def handler(func):
@@ -466,38 +487,45 @@ def handler(func):
     def wrapper(handler, *cfg_args, **cfg_kwargs):
         # We'll create a (*args, **kwargs)
         # College args from the config args:, then pattern /(.*)/(.*)
+        req_args = list(cfg_args) + handler.path_args
         # Collect kwargs from the config kwargs:, then pattern /(?P<key>.*), then URL query params
-        all_args, all_kwargs = list(cfg_args), dict(cfg_kwargs)
-        all_kwargs.setdefault('handler', handler)
-        all_args.extend(handler.path_args)
-        all_kwargs.update(handler.path_kwargs)
-        all_kwargs.update(handler.args)
+        req_kwargs = {'handler': handler}
+        for d in (cfg_args, handler.path_kwargs, handler.args):
+            req_kwargs.update(d)
+        headers = handler.request.headers
         # If POSTed with Content-Type: application/json, parse body as well
-        if handler.request.headers.get('Content-Type', '') == 'application/json':
-            all_kwargs.update(json.loads(handler.request.body))
+        if headers.get('Content-Type', '') == 'application/json':
+            req_kwargs.update(json.loads(handler.request.body))
 
         # Map these into the signature
+        # TODO: Fix tests.test_functionhandler:TestWrapper
         args, kwargs = [], {}
         for arg, param in params.items():
             hint = hints.get(arg, identity)
-            # Populate positional arguments from all_args
-            if len(all_args):
+            # If hint says it's a header, pass the header without converting
+            if hint is Header:
+                kwargs[arg] = handler.request.headers.get(arg)
+                continue
+            # Populate positional arguments from req_args
+            # TODO: Document logic
+            if len(req_args):
                 if param.kind in {param.POSITIONAL_ONLY, param.POSITIONAL_OR_KEYWORD}:
-                    args.append(convert(hint, all_args.pop(0)))
+                    args.append(convert(hint, param, req_args.pop(0)))
                 elif param.kind == param.VAR_POSITIONAL:
-                    for val in all_args:
-                        args.append(convert(hint, val))
-                    all_args.clear()
-            # Populate keyword arguments from all_kwargs
-            if arg in all_kwargs:
+                    for val in req_args:
+                        args.append(convert(hint, param, val))
+                    req_args.clear()
+            # Populate keyword arguments from req_kwargs
+            if arg in req_kwargs:
                 if param.kind in {param.KEYWORD_ONLY, param.POSITIONAL_OR_KEYWORD}:
-                    kwargs[arg] = convert(hint, all_kwargs.pop(arg))
+                    kwargs[arg] = convert(hint, param, req_kwargs.pop(arg))
                 elif param.kind == param.VAR_POSITIONAL:
-                    for val in flatten([all_kwargs.pop(arg)]):
-                        args.append(convert(hint, val))
+                    for val in flatten([req_kwargs.pop(arg)]):
+                        args.append(convert(hint, param, val))
 
         return func(*args, **kwargs)
 
+    wrapper.__func__ = func
     return wrapper
 
 
@@ -554,6 +582,6 @@ def build_log_info(keys, *vars):
     code = compile('def fn(handler, %s):\n\treturn {%s}' % (', '.join(vars), ' '.join(vals)),
                    filename='log', mode='exec')
     context = {'os': os, 'time': time, 'datetime': datetime, 'conf': conf, 'AttrDict': AttrDict}
-    # The code is constructed entirely by this function. Using exec is safe
-    exec(code, context)         # nosec
+    # exec() is safe here since the code is constructed entirely in this function
+    exec(code, context)     # nosec: developer-initiated
     return context['fn']
