@@ -1,7 +1,6 @@
 import io
 import os
 import re
-import six
 import json
 import time
 import yaml
@@ -9,10 +8,8 @@ import unittest
 import gramex.cache
 import pandas as pd
 import sqlalchemy as sa
-from lxml import etree
 from gramex.cache import hashfn
 from gramex.config import variables, str_utf8
-from six import string_types
 from markdown import markdown
 from collections import OrderedDict
 from orderedattrdict import AttrDict
@@ -241,7 +238,9 @@ class TestOpen(unittest.TestCase):
     def test_open_yaml(self):
         path = os.path.join(cache_folder, 'data.yaml')
         with io.open(path, encoding='utf-8') as handle:
-            expected = yaml.load(handle, Loader=AttrDictYAMLLoader)
+            # yaml.load() is safe to use here since we're loading from a known safe file.
+            # Specifically, we're testing whether Loader= is passed to gramex.cache.open.
+            expected = yaml.load(handle, Loader=AttrDictYAMLLoader)     # nosec: test case
 
         def check(reload):
             result, reloaded = gramex.cache.open(path, 'yaml', _reload_status=True,
@@ -282,26 +281,12 @@ class TestOpen(unittest.TestCase):
         def check(reload):
             result, reloaded = gramex.cache.open(path, 'md', _reload_status=True, encoding='utf-8')
             eq_(reloaded, reload)
-            ok_(isinstance(result, six.text_type))
+            ok_(isinstance(result, str))
             eq_(result, expected)
 
         self.check_file_cache(path, check)
         eq_(gramex.cache.open(path), gramex.cache.open(path, 'md'))
 
-    def test_open_xml(self):
-        path = os.path.join(cache_folder, 'data.svg')
-        expected = etree.parse(path)
-
-        def check(reload):
-            result, reloaded = gramex.cache.open(path, 'svg', _reload_status=True)
-            eq_(reloaded, reload)
-            eq_(etree.tostring(result), etree.tostring(expected))
-
-        self.check_file_cache(path, check)
-        for ext in ['xml', 'svg', 'rss', 'atom']:
-            eq_(etree.tostring(gramex.cache.open(path)),
-                etree.tostring(gramex.cache.open(path, ext)))
-
     def test_open_custom(self):
         def img_size(path, scale=1):
             return tuple(v * scale for v in Image.open(path).size)
@@ -426,7 +411,7 @@ class TestOpen(unittest.TestCase):
         data = gramex.cache.open(path, 'csv')
         ok_(isinstance(data, pd.DataFrame))
         text = gramex.cache.open(path, 'text')
-        ok_(isinstance(text, string_types))
+        ok_(isinstance(text, str))
         none = gramex.cache.open(path, lambda path: None)
         ok_(none is None)
 
