@@ -1,23 +1,29 @@
 '''Test case utilities'''
 import os
 import csv
-import six
 import sys
 import json
 import time
 import random
 import pandas as pd
+import numpy as np
+from io import StringIO
 from collections import Counter
 from orderedattrdict import AttrDict
+from typing import List
+from typing_extensions import Annotated
 from sklearn.datasets import make_circles as sk_make_circles
 from tornado import gen
-from tornado.web import RequestHandler, MissingArgumentError
+from tornado.web import RequestHandler, MissingArgumentError, HTTPError
+from tornado.gen import coroutine
 from tornado.httpclient import AsyncHTTPClient
 from concurrent.futures import ThreadPoolExecutor
 from gramex.cache import Subprocess
 from gramex.services import info
 from gramex.services.emailer import SMTPStub
 from gramex.handlers import BaseHandler
+from gramex.http import OK
+from gramex.transforms import handler, Header
 
 watch_info = []
 ws_info = []
@@ -213,7 +219,7 @@ def log_format(handler):
 
 
 def log_csv(handler):
-    result = six.StringIO()
+    result = StringIO()
     writer = csv.writer(result)
     try:
         1 / 0
@@ -265,7 +271,7 @@ def increment_header(handler):
     Does not return any output. Used as a FunctionHandler in func/redirect
     '''
     counters['header'] += 1
-    handler.set_header('Increment', six.text_type(counters['header']))
+    handler.set_header('Increment', str(counters['header']))
     return 'Constant result'
 
 
@@ -310,15 +316,16 @@ def subprocess(handler):
 def argparse(handler):
     params = json.loads(handler.get_argument('_q'))
     args = params.get('args', [])
+    typemap = {'list': list, 'str': str, 'None': None, 'int': int, 'bool': bool}
     # Convert first parameter to relevant type, if required
     if len(args) > 0:
-        if args[0] in {'list', 'str', 'unicode', 'six.string_type', 'None'}:
-            args[0] = eval(args[0])
+        if args[0] in typemap:
+            args[0] = typemap[args[0]]
     # Convert type: to a Python class
     kwargs = params.get('kwargs', {})
-    for key, val in kwargs.items():
+    for val in kwargs.values():
         if 'type' in val:
-            val['type'] = eval(val['type'])
+            val['type'] = typemap[val['type']]
     return json.dumps(handler.argparse(*args, **kwargs))
 
 
@@ -473,6 +480,44 @@ def transform_circles(df, *argss, **kwargs):
     return df
 
 
+@coroutine
+def pynode_run(handler):
+    from gramex.pynode import node
+    kwargs = {}
+    for key, vals in handler.args.items():
+        try:
+            kwargs[key] = float(vals[-1])
+        except ValueError:
+            kwargs[key] = vals[-1]
+    result = yield node.js(**kwargs)
+    return result
+
+
+@handler
+def test_function(
+        li1: List[int],
+        lf1: List[float],
+        li2: Annotated[List[int], 'List of ints'],  # noqa
+        lf2: Annotated[List[float], 'List of floats'],  # noqa
+        li3: List[int] = [0],
+        lf3: List[float] = [0.0],
+        l1=[],
+        i1: Annotated[int, 'First value'] = 0,  # noqa
+        i2: Annotated[int, 'Second value'] = 0,  # noqa
+        s1: str = 'Total',
+        n1: np.int = 0,
+        n2: np.int64 = 0,
+        h: Header = '',
+        code: int = OK):
+    '''
+    This is a **Markdown** docstring.
+    '''
+    if code == OK:
+        return json.dumps([li1, li2, li3, lf1, lf2, lf3, l1, i1, i2, s1, h])
+    else:
+        raise HTTPError(code, reason='Something')
+
+
 if __name__ == '__main__':
     # Call the method mentioned in the command line
     method_name = sys.argv[1]
